@@ -1,6 +1,6 @@
 /*
   (C) Nelson Murilo - 2004/09/13
-  Version 0.9
+  Version 0.10
   C port from chkproc.pl code from Klaus Steding-Jessen <jessen@nic.br>
   and Cristine Hoepers <cristine@nic.br> +little output changes.
 
@@ -40,12 +40,23 @@
   2005/10/28 - Bug fix for FreeBSD: chkproc was sending a SIGXFSZ (kill -25)
                to init, causing a reboot.  Patch by Nelson Murilo.
                Thanks to Luiz E. R. Cordeiro.
+
+  2005/11/15 - Add check for Enye LKM - Nelson Murilo
+
+  2005/11/25 - Fix for long lines in PS output - patch by Lantz Moore
+  
+  2006/01/05 - Add getpriority to identify LKMs, ideas from Yjesus(unhide) and 
+               Slider/Flimbo (skdet)
+
+  2006/01/11 - Fix signal 25 on parisc linux and return of kill() -
+               Thanks to Lantz Moore
 */
 
 #if !defined(__linux__) && !defined(__FreeBSD__) && !defined(__sun)
 int main (){ return 0; }
 #else
 #include <stdio.h>
+#include <string.h> 
 #include <errno.h>
 #include <sys/types.h>
 #include <dirent.h>
@@ -57,6 +68,7 @@ int main (){ return 0; }
 #include <procfs.h>
 #include <fcntl.h>
 #endif
+#include <sys/resource.h>
 
 #define PS_SUN 0
 #define PS_LOL 1
@@ -72,6 +84,10 @@ int main (){ return 0; }
 #define MAX_PROCESSES 99999
 #define MAX_BUF 1024
 
+#if !defined (SIGXFSZ)
+#define SIGXFSZ 25
+#endif 
+
 static char *ps_cmds[] = {
 	    "ps -edf",
 	    "ps auxw",
@@ -85,6 +101,30 @@ int dirproc[MAX_PROCESSES+1];
 int isathread[MAX_PROCESSES+1];
 #endif
 
+/*
+ * read at most the first (size-1) chars into s and terminate with a '\0'.
+ * stops reading after a newline or EOF.  if a newline is read, it will be
+ * the last char in the string.  if no newline is found in the first
+ * (size-1) chars, then keep reading and discarding chars until a newline
+ * is found or EOF.
+ */
+char *readline(char *s, int size, FILE *stream)
+{
+  char *rv = fgets(s, size, stream);
+
+  if (strlen(s) == (size-1) && s[size-1] != '\n')
+  {
+    char buf[MAX_BUF];
+    fgets(buf, MAX_BUF, stream);
+    while (strlen(buf) == (MAX_BUF-1) && buf[MAX_BUF-1] != '\n')
+    {
+      fgets(buf, MAX_BUF, stream);
+    }
+  }
+
+  return rv;
+}
+
 int main(int argc, char **argv)
 {
    char buf[MAX_BUF], *p, path[MAX_BUF];
@@ -102,7 +142,7 @@ int main(int argc, char **argv)
    psinfo_t psbuf;
 #endif
 
-   pv = verbose = 0;
+   pv = verbose = 0; 
 
    if (!proc)
    {
@@ -120,13 +160,15 @@ int main(int argc, char **argv)
       }
 #if defined(__linux__)
       else if (!memcmp(argv[i], "-p", 2))
-        if (i+1 < argc)
-           pv = atoi(argv[++i]);
-        else
-        {
-	   printf("Usage: %s [-v] [-v] [-p procps version]\n", argv[0]);
-	   return 0;
-        }
+      {
+         if (i+1 < argc) 
+            pv = atoi(argv[++i]);
+         else
+         { 
+	    printf("Usage: %s [-v] [-v] [-p procps version]\n", argv[0]);
+	    return 0;
+         }
+      }
 #endif
    }
 #if defined(__sun)
@@ -141,7 +183,7 @@ int main(int argc, char **argv)
 /*  printf("pv = %d\n\r", pv); /* -- DEBUG */
 #endif
 
-/* printf("pscmd = %s\n\r", pscmd); /* -- DEBUG */
+/* printf("pscmd = %s\n\r", pscmd); /* -- DEBUG */ 
    if (!(ps = popen(pscmd, "r")))
    {
        perror("ps");
@@ -149,14 +191,14 @@ int main(int argc, char **argv)
    }
 
    *buf = 0;
-   fgets(buf, MAX_BUF, ps); /* Skip header */
+   readline(buf, MAX_BUF, ps); /* Skip header */
 #if defined(__sun)
    if (!isspace(*buf))
 #else
    if (!isalpha(*buf))
 #endif
    {
-     fgets(buf, MAX_BUF, ps); /* Skip header */
+     readline(buf, MAX_BUF, ps); /* Skip header */
      if (!isalpha(*buf) && pv != PS_LNX)
      {
 	if (pv != PS_LOL)
@@ -175,7 +217,7 @@ int main(int argc, char **argv)
 #endif
    }
 
-   while (fgets(buf, MAX_BUF, ps))
+   while (readline(buf, MAX_BUF, ps))
    {
       p = buf;
 #if defined(__sun)
@@ -190,7 +232,7 @@ int main(int argc, char **argv)
       ret = atol(p);
       if ( ret < 0 || ret > MAX_PROCESSES )
       {
-         fprintf (stderr, " OooPS, not expected %d value\n", ret);
+         fprintf (stderr, " OooPS, not expected %ld value\n", ret);
          exit (2);
       }
       psproc[ret] = 1;
@@ -217,7 +259,7 @@ int main(int argc, char **argv)
       else if (maybeathread) {
          isathread[atol(tmp_d_name)] = 1; /* mark it as a linux NTPL thread if it's in the form of "\.[0-9]*" */
          if (verbose)
-            printf("%d is a Linux Thread, marking as such...\n", atol(tmp_d_name));
+            printf("%ld is a Linux Thread, marking as such...\n", atol(tmp_d_name));
       }
 #endif
 
@@ -234,7 +276,6 @@ int main(int argc, char **argv)
       snprintf(&buf[6], 6, "%d", i);
       if (!chdir(buf))
       {
-/*         if (!dirproc[i] ) /*  && !kill(i, 0)) */
          if (!dirproc[i] && !psproc[i])
          {
 #if defined(__linux__)
@@ -293,6 +334,18 @@ int main(int argc, char **argv)
          }
 #endif
       }
+      else 
+      {
+         errno = 0;
+         getpriority(PRIO_PROCESS, i);
+         if (!errno)
+         {
+            retdir++;
+            if (verbose)
+	       printf ("PID %5d(%s): not in getpriority readdir output\n", i, buf);
+	 }
+      }
+
    }
    if (retdir)
       printf("You have % 5d process hidden for readdir command\n", retdir);
@@ -300,11 +353,17 @@ int main(int argc, char **argv)
       printf("You have % 5d process hidden for ps command\n", retps);
 #if defined(__linux__)
    kill(1, 100); /*  Check for SIGINVISIBLE Adore signal */
-   if (kill (1, 25) > 0 && errno == 3)
+   if (kill (1, SIGXFSZ) < 0  && errno == 3)
    {
       printf("SIGINVISIBLE Adore found\n");
       retdir+= errno;
    }
+   /* Check for Enye LKM */
+   if (kill (12345, 58) >= 0)
+   {
+      printf("Enye LKM found\n");
+      retdir+= errno;
+   }
 #endif
    return (retdir+retps);
 }
