@@ -303,9 +303,9 @@ static int if_fetch(char *ifname, struct interface *ife)
   struct ifreq ifr;
 
   memset((char *) ife, 0, sizeof(struct interface));
-  strcpy(ife->name, ifname);
+  strncpy(ife->name, ifname, sizeof(ife->name));
 
-  strcpy(ifr.ifr_name, ifname);
+  strncpy(ifr.ifr_name, ifname, sizeof(ifr.ifr_name));
   if (ioctl(skfd, SIOCGIFFLAGS, &ifr) < 0)
       return(-1);
   ife->flags = ifr.ifr_flags;
